@@ -1,8 +1,9 @@
 package uk.ac.starlink.ttools.join;
 
 import java.io.IOException;
-import java.util.logging.Logger;
+import java.util.Collection;
 import uk.ac.starlink.table.JoinFixAction;
+import uk.ac.starlink.table.RowRunner;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.Tables;
 import uk.ac.starlink.table.join.LinkSet;
@@ -10,6 +11,7 @@ import uk.ac.starlink.table.join.MatchEngine;
 import uk.ac.starlink.table.join.MatchStarTables;
 import uk.ac.starlink.table.join.MultiJoinType;
 import uk.ac.starlink.table.join.ProgressIndicator;
+import uk.ac.starlink.table.join.RowLink;
 import uk.ac.starlink.table.join.RowMatcher;
 import uk.ac.starlink.task.ChoiceParameter;
 import uk.ac.starlink.task.Environment;
@@ -38,13 +40,11 @@ public class MatchMapper implements TableMapper {
     private final ChoiceParameter<String> mmodeParam_;
     private final IntegerParameter irefParam_;
     private final ProgressIndicatorParameter progressParam_;
+    private final Parameter<RowRunner> runnerParam_;
 
     private static final String PAIRS_MODE = "pairs";
     private static final String GROUP_MODE = "group";
 
-    private static final Logger logger =
-        Logger.getLogger( "uk.ac.starlink.ttools.join" );
-
     /**
      * Constructor.
      */
@@ -114,6 +114,7 @@ public class MatchMapper implements TableMapper {
         matcherParam_ = new MatchEngineParameter( "matcher" );
         fixcolsParam_ = new JoinFixActionParameter( "fixcols" );
         progressParam_ = new ProgressIndicatorParameter( "progress" );
+        runnerParam_ = new MatchRunnerParameter( "runner" );
     }
 
     public Parameter<?>[] getParameters() {
@@ -128,6 +129,7 @@ public class MatchMapper implements TableMapper {
             fixcolsParam_,
             fixcolsParam_.createSuffixParameter( "N" ),
             progressParam_,
+            runnerParam_,
         };
     }
 
@@ -162,13 +164,14 @@ public class MatchMapper implements TableMapper {
         }
         ProgressIndicator progger =
             progressParam_.progressIndicatorValue( env );
+        RowRunner runner = runnerParam_.objectValue( env );
         if ( GROUP_MODE.equalsIgnoreCase( mmode ) ) {
             return new GroupMatchMapping( matcher, exprTuples, fixActs, progger,
-                                          joinTypes );
+                                          runner, joinTypes );
         }
         else if ( PAIRS_MODE.equalsIgnoreCase( mmode ) ) {
             return new PairsMatchMapping( matcher, exprTuples, fixActs, progger,
-                                          iref, joinTypes );
+                                          runner, iref, joinTypes );
         }
         else {
             throw new AssertionError( "Unknown multimode " + mmode + "???" );
@@ -185,6 +188,7 @@ public class MatchMapper implements TableMapper {
         private final String[][] exprTuples_;
         private final JoinFixAction[] fixActs_;
         private final ProgressIndicator progger_;
+        private final RowRunner runner_;
 
         /**
          * Constructor.
@@ -195,13 +199,17 @@ public class MatchMapper implements TableMapper {
          * @param   fixActs   nin-element array of actions for fixing up 
          *                    duplicated table columns
          * @param   progger   progress indicator
+         * @param   runner    controls parallel implementation,
+         *                    or null for sequential
          */
         MatchMapping( MatchEngine matchEngine, String[][] exprTuples,
-                      JoinFixAction[] fixActs, ProgressIndicator progger ) {
+                      JoinFixAction[] fixActs, ProgressIndicator progger,
+                      RowRunner runner ) {
             matchEngine_ = matchEngine;
             exprTuples_ = exprTuples;
             fixActs_ = fixActs;
             progger_ = progger;
+            runner_ = runner;
             nin_ = exprTuples_.length;
         }
 
@@ -229,22 +237,21 @@ public class MatchMapper implements TableMapper {
             }
 
             /* Do the match. */
-            RowMatcher matcher = new RowMatcher( matchEngine_, subTables );
+            RowMatcher matcher =
+                RowMatcher.createMatcher( matchEngine_, subTables, runner_ );
             matcher.setIndicator( progger_ );
             LinkSet matches;
             try { 
                 matches = findMatches( matcher );
-                if ( ! matches.sort() ) {
-                    logger.warning( "Implementation can't sort rows - "
-                                  + "matched table rows may not be sorted" );
-                }
             }
             catch ( InterruptedException e ) {
                 throw new ExecutionException( e.getMessage(), e );
             }
 
             /* Create a new table based on the matched rows. */
-            return createJoinTable( inTables, matches, fixActs_ );
+            Collection<RowLink> orderedMatches =
+                MatchStarTables.orderLinks( matches );
+            return createJoinTable( inTables, orderedMatches, fixActs_ );
         }
 
         /**
@@ -265,9 +272,10 @@ public class MatchMapper implements TableMapper {
          * @param   matches    row link set
          * @param   fixActs    actions for fixing up duplicated table columns
          */
-        protected abstract StarTable createJoinTable( StarTable[] inTables,
-                                                      LinkSet matches,
-                                                      JoinFixAction[] fixActs )
+        protected abstract StarTable
+                createJoinTable( StarTable[] inTables,
+                                 Collection<RowLink> matches,
+                                 JoinFixAction[] fixActs )
                 throws IOException;
       
 
@@ -310,13 +318,16 @@ public class MatchMapper implements TableMapper {
          * @param   fixActs   nin-element array of actions for fixing up 
          *                    duplicated table columns
          * @param   progger   progress indicator
+         * @param   runner    controls parallel implementation,
+         *                    or null for sequential
          * @param   iref      index (0-based) of reference table
          * @param   joinTypes inclusion criteria for links in output table
          */
         PairsMatchMapping( MatchEngine matchEngine, String[][] exprTuples,
                            JoinFixAction[] fixActs, ProgressIndicator progger,
+                           RowRunner runner,
                            int iref, MultiJoinType[] joinTypes ) {
-            super( matchEngine, exprTuples, fixActs, progger );
+            super( matchEngine, exprTuples, fixActs, progger, runner );
             iref_ = iref;
             joinTypes_ = joinTypes;
         }
@@ -327,7 +338,7 @@ public class MatchMapper implements TableMapper {
         }
 
         protected StarTable createJoinTable( StarTable[] inTables,
-                                             LinkSet matches,
+                                             Collection<RowLink> matches,
                                              JoinFixAction[] fixActs ) {
             return MatchStarTables
                   .makeJoinTable( inTables, matches, false, fixActs, null );
@@ -350,12 +361,14 @@ public class MatchMapper implements TableMapper {
          * @param   fixActs   nin-element array of actions for fixing up 
          *                    duplicated table columns
          * @param   progger   progress indicator
+         * @param   runner    controls parallel implementation,
+         *                    or null for sequential
          * @param   joinTypes inclusion criteria for links in output table
          */
         GroupMatchMapping( MatchEngine matchEngine, String[][] exprTuples,
                            JoinFixAction[] fixActs, ProgressIndicator progger,
-                           MultiJoinType[] joinTypes ) {
-            super( matchEngine, exprTuples, fixActs, progger );
+                           RowRunner runner, MultiJoinType[] joinTypes ) {
+            super( matchEngine, exprTuples, fixActs, progger, runner );
             joinTypes_ = joinTypes;
         }
 
@@ -365,7 +378,7 @@ public class MatchMapper implements TableMapper {
         }
 
         protected StarTable createJoinTable( StarTable[] inTables,
-                                             LinkSet matches,
+                                             Collection<RowLink> matches,
                                              JoinFixAction[] fixActs ) {
             return MatchStarTables
                   .makeJoinTable( inTables, matches, false, fixActs, null );
