@@ -233,6 +233,26 @@ public abstract class CombineArrayPlotter
         return xyData;
     }
 
+    /**
+     * Returns a comment about suitability for non-uniformly sampled spectra.
+     *
+     * @return  XML text suitable for insertion in description text
+     */
+    static String getXYCombineComment() {
+        return String.join( "\n",
+            "<p>Note that because the X and Y arrays must be of a fixed size",
+            "for all rows, and because combination is performed in both",
+            "X and Y directions,",
+            "this is typically only suitable for plotting combined spectra",
+            "if they all share a common horizontal axis,",
+            "e.g. are all sampled into the same wavelength bins.",
+            "To visually combine spectra with non-uniform sampling,",
+            "the <ref id='layer-arrayquantile'>arrayquantile</ref> plotter",
+            "may be more useful.",
+            "</p>",
+        "" );
+    }
+
     /**
      * Partial Style implementation for use with this class.
      */
@@ -460,16 +480,12 @@ public abstract class CombineArrayPlotter
                             return;
                         }
                         if ( hasX_ ) {
-                            double[] xs = XS_COORD.readArrayCoord( tseq, IC_XS);
-                            for ( int ip = 0; ip < npx; ip++ ) {
-                                xyData.xBins_.submitToBin( ip, xs[ ip ] );
-                            }
+                            submitArray( XS_COORD.readArrayCoord( tseq, IC_XS ),
+                                         npx, xyData.xBins_ );
                         }
                         if ( hasY_ ) {
-                            double[] ys = YS_COORD.readArrayCoord( tseq, IC_YS);
-                            for ( int ip = 0; ip < npy; ip++ ) {
-                                xyData.yBins_.submitToBin( ip, ys[ ip ] );
-                            }
+                            submitArray( YS_COORD.readArrayCoord( tseq, IC_YS ),
+                                         npy, xyData.yBins_ );
                         }
                     }
                     else {
@@ -548,6 +564,24 @@ public abstract class CombineArrayPlotter
                 xyData.yBins_ = null;
             }
         }
+
+        /**
+         * Adds an array of samples to the corresponding bins of a given
+         * bin list.
+         *
+         * @param  samples  data array
+         * @param  np    size of data array
+         * @param  binList  list of bins for data accumulation
+         */
+        private static void submitArray( double[] samples, int np,
+                                         BinList binList ) {
+            for ( int ip = 0; ip < np; ip++ ) {
+                double d = samples[ ip ];
+                if ( ! Double.isNaN( d ) ) {
+                    binList.submitToBin( ip, d );
+                }
+            }
+        }
     }
 
     /**
