@@ -73,6 +73,12 @@ public abstract class LabelPlotter extends AbstractPlotter<LabelStyle> {
         , true );
     private static final int MAX_CROWDLIMIT = Byte.MAX_VALUE / 2 - 1;
 
+    /** Config key to control X pixel offset. */
+    public static final ConfigKey<Integer> XOFF_KEY = createOffsetKey( "X" );
+
+    /** Config key to control Y pixel offset. */
+    public static final ConfigKey<Integer> YOFF_KEY = createOffsetKey( "Y" );
+
     /** Config key to control minimum pixel label spacing. */
     public static final ConfigKey<Integer> SPACING_KEY =
         IntegerConfigKey.createSliderKey(
@@ -189,6 +195,8 @@ public abstract class LabelPlotter extends AbstractPlotter<LabelStyle> {
         list.addAll( Arrays.asList( new ConfigKey<?>[] {
             StyleKeys.ANCHOR,
             StyleKeys.COLOR,
+            XOFF_KEY,
+            YOFF_KEY,
             SPACING_KEY,
             CROWDLIMIT_KEY,
         } ) );
@@ -204,10 +212,12 @@ public abstract class LabelPlotter extends AbstractPlotter<LabelStyle> {
         }
         byte crowdLimit = (byte) iclimit;
         assert crowdLimit == iclimit;
+        Point offset = new Point( config.get( XOFF_KEY ).intValue(),
+                                  - config.get( YOFF_KEY ).intValue() );
         return new LabelStyle( CAPTIONER_KEYSET.createValue( config ),
                                config.get( StyleKeys.ANCHOR ),
                                config.get( StyleKeys.COLOR ),
-                               config.get( SPACING_KEY ), crowdLimit );
+                               config.get( SPACING_KEY ), crowdLimit, offset );
     }
 
     public PlotLayer createLayer( DataGeom geom0,
@@ -468,6 +478,28 @@ public abstract class LabelPlotter extends AbstractPlotter<LabelStyle> {
         };
     }
 
+    /**
+     * Returns a config key for specifying pixel offset in a named direction.
+     *
+     * @param  axName  axis name
+     * @return  config key
+     */
+    private static ConfigKey<Integer> createOffsetKey( String axisName ) {
+        String axName = axisName.toUpperCase();
+        String axname = axisName.toLowerCase();
+        ConfigMeta meta = new ConfigMeta( axname + "off", axName + " Offset" );
+        meta.setStringUsage( "<pixels>" );
+        meta.setShortDescription( "Pixel offset in " + axName + " direction" );
+        meta.setXmlDescription( new String[] {
+            "<p>Allows fine adjustment of label positioning in the",
+            axName + " direction.",
+            "The value is a positive or negative pixel offset",
+            "applied to the position of each plotted label.",
+            "</p>",
+        } );
+        return IntegerConfigKey.createSpinnerKey( meta, 0, -999, 999 );
+    }
+
     /**
      * Creates an instance of this class for use with Area plotters.
      * The position is acquired from the area values not explicitly
@@ -809,11 +841,8 @@ public abstract class LabelPlotter extends AbstractPlotter<LabelStyle> {
         }
 
         public Pixer createPixer( Rectangle clip ) {
-            Anchor anchor = style_.getAnchor();
-            Captioner captioner = style_.getCaptioner();
             Caption caption = Caption.createCaption( label_ );
-            Rectangle labelBox =
-                anchor.getCaptionBounds( caption, 0, 0, captioner );
+            Rectangle labelBox = style_.getCaptionBounds( caption );
             Rectangle drawBox = labelBox.intersection( clip );
             if ( drawBox.isEmpty() ) {
                 return null;
@@ -823,8 +852,8 @@ public abstract class LabelPlotter extends AbstractPlotter<LabelStyle> {
 
             /* We don't do anything clever with antialiased text. */
             Graphics g = bitmap.getImage().createGraphics();
-            anchor.drawCaption( caption, -labelBox.x, -labelBox.y,
-                                captioner, g );
+            g.translate( -labelBox.x, -labelBox.y );
+            style_.drawLabel( g, caption );
             return Pixers.translate( bitmap.createPixer(),
                                      drawBox.x, drawBox.y );
         }
