@@ -107,8 +107,10 @@ public class ShapePlotter extends AbstractPlotter<ShapeStyle>
         Outliner outliner = style.getOutliner();
         Stamper stamper = style.getStamper();
         geom = form_.adjustGeom( geom );
-        return mode_.createLayer( this, form_, geom, dataSpec,
-                                  outliner, stamper );
+        return outliner.canPaint( dataSpec )
+             ? mode_.createLayer( this, form_, geom, dataSpec,
+                                  outliner, stamper )
+             : null;
     }
 
     /**
